"use client"

import { updateCommissionType } from "@/actions/items/commissions/types/updateType";
import { Button } from "@/components/ui/button";
import { Form, FormControl, FormDescription, FormField, FormItem, FormLabel, FormMessage } from "@/components/ui/form";
import { Input } from "@/components/ui/input";
import { CommissionExtra, CommissionOption, CommissionType, CommissionTypeExtra, CommissionTypeOption } from "@/generated/prisma";
import { commissionTypeSchema } from "@/schemas/commissionType";
import { zodResolver } from "@hookform/resolvers/zod";
import { useRouter } from "next/navigation";
import { useForm } from "react-hook-form";
import { toast } from "sonner";
import * as z from "zod/v4";
import { CommissionExtraField } from "./form/CommissionExtraField";
import { CommissionOptionField } from "./form/CommissionOptionField";

type CommissionTypeWithConnections = CommissionType & {
  options: (CommissionTypeOption & { option: CommissionOption })[]
  extras: (CommissionTypeExtra & { extra: CommissionExtra })[]
}

type Props = {
  type: CommissionTypeWithConnections
  allOptions: CommissionOption[],
  allExtras: CommissionExtra[],
}

export default function EditTypeForm({ type, allOptions, allExtras }: Props) {
  const router = useRouter();
  const form = useForm<z.infer<typeof commissionTypeSchema>>({
    resolver: zodResolver(commissionTypeSchema),
    defaultValues: {
      name: type.name,
      description: type.description ?? "",
      options: type.options.map((o) => ({
        optionId: o.optionId,
        price: o.price ?? undefined,
        pricePercent: o.pricePercent ?? undefined,
        priceRange: o.priceRange ?? undefined,
      })),
      extras: type.extras.map((e) => ({
        extraId: e.extraId,
        price: e.price ?? undefined,
        pricePercent: e.pricePercent ?? undefined,
        priceRange: e.priceRange ?? undefined,
      })),
    },
  })

  async function onSubmit(values: z.infer<typeof commissionTypeSchema>) {
    try {
      await updateCommissionType(type.id, values)
      toast.success("Commission type updated.")
      router.push("/items/commissions/types")
    } catch (err) {
      console.error(err)
      toast("Failed to create commission type.")
    }
  }

  return (
    <div className="flex flex-col gap-8">
      <Form {...form}>
        <form onSubmit={form.handleSubmit(onSubmit)} className="space-y-8">
          <FormField
            control={form.control}
            name="name"
            render={({ field }) => (
              <FormItem>
                <FormLabel>Name</FormLabel>
                <FormControl>
                  <Input {...field} />
                </FormControl>
                <FormDescription>The name of the commission type.</FormDescription>
                <FormMessage />
              </FormItem>
            )}
          />
          <FormField
            control={form.control}
            name="description"
            render={({ field }) => (
              <FormItem>
                <FormLabel>Description</FormLabel>
                <FormControl>
                  <Input {...field} />
                </FormControl>
                <FormDescription>Optional description.</FormDescription>
                <FormMessage />
              </FormItem>
            )}
          />

          <CommissionOptionField options={allOptions} />
          <CommissionExtraField extras={allExtras} />

          <div className="flex flex-col gap-4">
            <Button type="submit">Submit</Button>
            <Button type="reset" variant="secondary" onClick={() => router.back()}>Cancel</Button>
          </div>
        </form>
      </Form>
    </div>
  );
}