"use client"

import { updateCommissionTypeSortOrder } from "@/actions/items/commissions/types/updateCommissionTypeSortOrder";
import SortableItemCard from "@/components/drag/SortableItemCard";
import { Card, CardContent, CardDescription, CardFooter, CardHeader, CardTitle } from "@/components/ui/card";
import { CommissionExtra, CommissionOption, CommissionType, CommissionTypeExtra, CommissionTypeOption } from "@/generated/prisma";
import {
  closestCenter,
  DndContext,
  DragEndEvent,
  PointerSensor,
  useSensor,
  useSensors,
} from "@dnd-kit/core";
import {
  arrayMove,
  rectSortingStrategy,
  SortableContext
} from "@dnd-kit/sortable";
import { PencilIcon } from "lucide-react";
import Link from "next/link";
import { useEffect, useState } from "react";

type CommissionTypeWithItems = CommissionType & {
  options: (CommissionTypeOption & {
    option: CommissionOption | null
  })[]
  extras: (CommissionTypeExtra & {
    extra: CommissionExtra | null
  })[]
}

export default function ListTypes({ types }: { types: CommissionTypeWithItems[] }) {
  const [items, setItems] = useState(types)
  const [isMounted, setIsMounted] = useState(false)

  useEffect(() => {
    setIsMounted(true)
  }, [])

  const sensors = useSensors(useSensor(PointerSensor))

  const handleDragEnd = async (event: DragEndEvent) => {
    const { active, over } = event

    if (!over || active.id === over.id) return

    const oldIndex = items.findIndex((i) => i.id === active.id)
    const newIndex = items.findIndex((i) => i.id === over.id)

    if (oldIndex !== -1 && newIndex !== -1) {
      const newItems = arrayMove(items, oldIndex, newIndex)
      setItems(newItems)

      await updateCommissionTypeSortOrder(newItems.map((item, i) => ({ id: item.id, sortIndex: i })))
    }
  }

  if (!isMounted) return null

  return (
    <div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-4 gap-4">
      <DndContext sensors={sensors} collisionDetection={closestCenter} onDragEnd={handleDragEnd}>
        <SortableContext items={items.map((i) => i.id)} strategy={rectSortingStrategy}>
          {items.map(type => (
            <SortableItemCard key={type.id} id={type.id}>
              <Card>
                <CardHeader>
                  <CardTitle className="text-xl truncate">{type.name}</CardTitle>
                  <CardDescription>{type.description}</CardDescription>

                </CardHeader>
                <CardContent className="flex flex-col justify-start gap-4">
                  <div>
                    <h4 className="font-semibold">Options</h4>
                    <ul className="pl-4 list-disc">
                      {type.options.map((opt) => (
                        <li key={opt.id}>
                          {opt.option?.name}:{" "}
                          {opt.price !== null
                            ? `${opt.price}€`
                            : opt.pricePercent
                              ? `+${opt.pricePercent}%`
                              : opt.priceRange
                                ? `${opt.priceRange}€`
                                : "Included"}
                        </li>
                      ))}
                    </ul>
                  </div>
                  <div>
                    <h4 className="font-semibold">Extras</h4>
                    <ul className="pl-4 list-disc">
                      {type.extras.map((ext) => (
                        <li key={ext.id}>
                          {ext.extra?.name}:{" "}
                          {ext.price !== null
                            ? `${ext.price}€`
                            : ext.pricePercent
                              ? `+${ext.pricePercent}%`
                              : ext.priceRange
                                ? `${ext.priceRange}€`
                                : "Included"}
                        </li>
                      ))}
                    </ul>
                  </div>
                </CardContent>
                <CardFooter className="flex items-center justify-between">
                  <Link
                    href={`/items/commissions/types/${type.id}/edit`}
                    className="text-sm text-primary hover:underline flex items-center gap-1"
                  >
                    <PencilIcon className="h-4 w-4" />
                    Edit
                  </Link>
                </CardFooter>
              </Card>
            </SortableItemCard >
          ))}
        </SortableContext>
      </DndContext>
    </div>
  );
}